# -*- coding: utf-8 -*-
"""
migrations package.
"""

from pyrin.packaging.base import Package


class MigrationsPackage(Package):
    """
    migrations package class.
    """

    NAME = __name__
    ENABLED = False
