# -*- coding: utf-8 -*-
"""
database orm package.
"""

from pyrin.packaging.base import Package


class DatabaseORMPackage(Package):
    """
    database orm package class.
    """

    NAME = __name__
    DEPENDS = []
